use clap::Parser;
use lettre::message::Mailbox;
use lettre::message::header::ContentType;
use lettre::transport::smtp::authentication::Credentials;
use lettre::transport::smtp::extension::ClientId;
use lettre::{Message, SmtpTransport, Transport};
use tracing::Level;
use tracing_subscriber::FmtSubscriber;

/// Simple mail sender
#[derive(Parser, Debug)]
#[command(version, about, long_about = None)]
struct Args {
    /// Src mail
    #[arg(short, long)]
    from_mail: String,

    /// Src name
    #[arg(long)]
    from_name: Option<String>,

    /// Dst mail
    #[arg(long)]
    to_mail: String,

    /// Dst name
    #[arg(long)]
    to_name: Option<String>,

    /// Reply to mail
    #[arg(long)]
    reply_to_mail: Option<String>,

    /// Reply to name
    #[arg(long)]
    reply_to_name: Option<String>,

    /// Subject
    #[arg(long)]
    subject: String,

    /// Text body
    #[arg(long)]
    text_body: String,

    /// SMTP domain
    #[arg(long)]
    smtp_domain: String,

    /// SMTP port
    #[arg(long)]
    smtp_port: Option<u16>,

    /// SMTP username
    #[arg(long)]
    smtp_username: Option<String>,

    /// SMTP password
    #[arg(long)]
    smtp_password: Option<String>,

    /// Disable TLS
    #[arg(long)]
    disable_tls: bool,

    /// Relay name
    #[arg(long, default_value = "my-computer.com")]
    smtp_relay_name: String,
}

fn main() {
    let args = Args::parse();

    // a builder for `FmtSubscriber`.
    let subscriber = FmtSubscriber::builder()
        // all spans/events with a level higher than TRACE (e.g, debug, info, warn, etc.)
        // will be written to stdout.
        .with_max_level(Level::TRACE)
        // completes the builder.
        .finish();

    tracing::subscriber::set_global_default(subscriber)
        .expect("setting default subscriber failed");


    let mut builder = Message::builder();
    builder = builder.from(Mailbox::new(
        args.from_name,
        args.from_mail.parse().unwrap(),
    ));
    if let Some(reply_to_mail) = args.reply_to_mail {
        builder = builder.reply_to(Mailbox::new(
            args.reply_to_name,
            reply_to_mail.parse().unwrap(),
        ));
    }

    builder = builder
        .to(Mailbox::new(
            args.to_name.clone(),
            args.to_mail.parse().unwrap(),
        ))
        .subject(args.subject)
        .header(ContentType::TEXT_PLAIN);
    let email = builder.body(args.text_body).unwrap();

    // Open a remote connection to gmail
    let mut mailer = match args.disable_tls {
        true => SmtpTransport::builder_dangerous(&args.smtp_domain),
        false => SmtpTransport::relay(&args.smtp_domain).unwrap(),
    };
    mailer = mailer.hello_name(ClientId::Domain(args.smtp_relay_name));

    if let (Some(username), Some(password)) = (args.smtp_username, args.smtp_password) {
        mailer = mailer.credentials(Credentials::new(username, password));
    }

    if let Some(port) = args.smtp_port {
        mailer = mailer.port(port);
    }

    let mailer = mailer.build();

    // Send the email
    match mailer.send(&email) {
        Ok(d) => println!(
            "Email sent successfully! code={} message={:?}",
            d.code(),
            d.message().collect::<Vec<_>>()
        ),
        Err(e) => panic!("Could not send email: {e:?}"),
    }
}


#[cfg(test)]
mod test {
    use crate::Args;
    #[test]
    fn verify_cli() {
        use clap::CommandFactory;
        Args::command().debug_assert()
    }
}